{# -*- coding: utf-8 -*-

  This file is part of Invenio.
  Copyright (C) 2015-2020 CERN.

  Invenio is free software; you can redistribute it and/or modify it
  under the terms of the MIT License; see LICENSE file for more details.
#}

{%- extends "invenio_oauth2server/settings/index.html" %}

{%- import "invenio_oauth2server/settings/helpers.html" as helpers with context %}
{%- from "invenio_oauth2server/settings/_macros.html" import render_field, render_scopes_field with context %}

{% block settings_content %}
{{ helpers.panel_start(
    _('Personal access token / %(name)s',
    name=token.client.name),
    with_body=False,
) }}
{% if show_token %}
<div class="ui segment">
  <p>
    <strong>{{ _('Access token') }}</strong><br>
    <div class="ui small label">
      <code>{{ token.access_token }}</code>
    </div>
  </p>
  <div class="ui info message">
    <p><i class="bullhorn icon"></i> {{ _("Please copy the personal access token now. You won't see it again!") }}</p>
  </div>
  <div class="ui negative message">
    <p><i class="exclamation triangle icon"></i> {{ _('Do not share this personal access token. It gives full access to your account.') }}</p>
  </div>
  {%- block token_footer_text %}{%- endblock %}
</div>
{% endif %}
<div class="ui segment">
    {% if not show_token %}
    <div class="ui warning message">
      <p>{{ _("If you've lost or forgotten this token please create a new one. Be aware that any scripts or applications using this token will need to be updated.") }}</p>
    </div>
    {% endif %}
    <form class="ui form" method="POST" role="form" action="">
      {{ render_field(form.name, autofocus=True) }}
      {{ render_scopes_field(form.scopes) }}
      {{ form.csrf_token }}
      <button type="submit" name="delete" class="ui button negative">
        <i class="icon trash"></i> {{ _('Delete') }}
      </button>
      <button type="submit" name="save" class="ui button primary">
        <i class="icon check"></i> {{ _('Save') }}
      </button>
    </form>
</div>
{{helpers.panel_end(with_body=False)}}
{% endblock %}
